from browser import webcomponent


class Component:
    def __init__(self, tag, template):
        self.tag = tag
        self.template = template
        self.application = None

    def __call__(self, klass):
        setattr(klass, 'component_config', self)

        if webcomponent.get(self.tag) is None:
            webcomponent.define(self.tag, klass)

        return klass




def event_handler(fn):
    def handler(ev):
        el = ev.target
        while True:
            if hasattr(el, 'component_config'):
                break

            el = el.parent
            if el is None:
                break

        if el is not None:
            return fn(el)(ev)
        else:
            raise RuntimeError('No component parent')

    return handler



from browser import window

from kirui.core import createElement as h, Component, CustomElement
from django_brython.assets import require


@Component(
    tag='kr-textbox',
    template=require('text.html')
)
class TextBox(CustomElement):
    DEFAULT_HEIGHT = 480

    @property
    def css_class(self):
        retval = {'form-control': True}
        if self.props.get('error', '') != '':
            retval['is-invalid'] = True

        return retval

    @property
    def is_readonly(self):
        if int(self.props['disabled']) == 1:
            return 'true'

        return ''

    def connectedCallback(self):
        window.tinymce.remove(f'#{self.props["id"]} textarea')

        window.tinymce.init({
            'target': self.querySelector('textarea'),
            'readonly': self.props['disabled'] == '1',
            'branding': False,
            'menubar': False,
            'height': self.props.get('height', self.DEFAULT_HEIGHT) or self.DEFAULT_HEIGHT
        })

    @property
    def form_data(self):
        return {self.props['name']: window.tinymce.get(f'{self.props["id"]}').getContent()}



from io import StringIO

from django.test import TestCase

from kirui.core import createElement as h, render, Component, patch_dom_with_vdom
from kirui.components.forms.widgets.date import DatePicker


class DiffingTest(TestCase):
    def _r_pprint(self, element, out, indent):
        if indent is None:
            s = ''
            end=''
        else:
            s = indent * '  '
            end = '\n'
            indent += 1
        if hasattr(element, 'tagName'):
            attrs = ' '.join(f'{k}="{v}"' for k, v in element._attributes.items())
            print(f'{s}<{element.tagName} {attrs}>', file=out, end=end)
            for child in element.children:
                self._r_pprint(child, indent=indent, out=out)
            print(f'{s}</{element.tagName}>', file=out, end=end)
        else:
            print(f'{s}{element.content}', file=out, end=end)

    def pprint(self, element, indent=0):
        s = StringIO()
        self._r_pprint(element, out=s, indent=indent)
        return s.getvalue()

    def test_diff_base(self):
        comp = DatePicker()
        content = comp._generate_picker_content()
        el1 = render(content, component=comp)
        el2 = render(content, component=comp)
        patch_dom_with_vdom(el1, el2)
        patch_dom_with_vdom(el1, el2)
        print(self.pprint(el1))



from io import StringIO

from samon.elements import BaseElement
from samon.render import RenderedElement


class KrApp(BaseElement):
    def to_xml(self, io: StringIO, indent: int, rendered_element: RenderedElement):
        try:
            module, entrypoint = rendered_element.node_attributes.pop('entrypoint').rsplit('.', 1)
            with rendered_element.frame(io, indent):
                print('<script type="text/python">\n', file=io, end='')
                print('import importlib', file=io)
                print('from kirui.core import createElement as h', file=io)
                print(f'module = importlib.import_module("{module}")', file=io)
                print(f'entrypoint = getattr(module, "{entrypoint}")', file=io)

                call = "entrypoint("
                for k, v in rendered_element.node_attributes.items():
                    call += f'{k}="{v}", '

                call += f'template={rendered_element.serialize("psx")})'
                print(call, file=io)

                print('</script>', file=io)
                print("""<script type="text/javascript">""", file=io)
                print("""brython({debug: 0, pythonpath: ['/brython/'], 'indexedDB': true});""", file=io)  # TODO: settings.DEBUG
                print("""</script>""", file=io)
        except KeyError:  # just for backward compatibility
            with rendered_element.frame(io, indent):
                print('<script type="text/javascript">', file=io, end='')
                print(f'var ${self.xml_attrs["id"]}_data = `', file=io, end='')
                rendered_element.to_jsx(io)
                print('`;</script>', file=io)



import ast
import sys

from io import StringIO

from django.test import TestCase

from browser.document import Element, AnonymusElement
from kirui.core import createElement as h, render, Component
from samon.environment import Environment
from samon.parser import DefaultParser


class RenderingTest(TestCase):
    def _r_pprint(self, element, out, indent):
        if indent is None:
            s = ''
            end=''
        else:
            s = indent * '  '
            end = '\n'
            indent += 1
        if hasattr(element, 'tagName'):
            attrs = ' '.join(f'{k}="{v}"' for k, v in element.attributes.items())
            print(f'{s}<{element.tagName} {attrs}>', file=out, end=end)
            for child in element.children:
                self._r_pprint(child, indent=indent, out=out)
            print(f'{s}</{element.tagName}>', file=out, end=end)
        else:
            print(f'{s}{element.content}', file=out, end=end)

    def pprint(self, element, indent=0):
        s = StringIO()
        self._r_pprint(element, out=s, indent=indent)
        return s.getvalue()

    def require(self, template):
        parser = DefaultParser(environment=Environment(loader=None))
        template = parser.parse(source=template, template_name='test')
        return eval(ast.unparse(template.serialize(output='psx')))

    def test_render_base(self):
        el = h('div', {'attr1': 7, 'attr2': 'example'}, ['Text content'])
        root = render(el)

        self.assertEqual(root.tagName, 'DIV')
        self.assertDictEqual(root.attributes, {'attr1': '7', 'attr2': 'example'})
        self.assertEqual(len(root.children), 1)

    def test_render_recursion(self):
        el = h('div', {}, [h('div', {}, ['Test content'])])
        root = render(el)

        self.assertEqual(len(root.children), 1)
        self.assertEqual(len(root.children[0].children), 1)

    def test_component(self):
        @Component(
            tag='kr-example',
            template=h('template', {}, [h('div', {}, [])])
        )
        class Example(Element):
            pass

        el = h('kr-example', {}, [])
        root = render(el)

        self.assertIsInstance(root, Example)
        self.assertEqual(len(root.children), 1)
        self.assertEqual(root.children[0].tagName, 'DIV')

    def test_component_slot(self):
        @Component(
            tag='kr-example',
            template=h('template', {}, [h('div', {}, [h('slot', {}, [])])])
        )
        class Example(Element):
            pass

        el = h('kr-example', {}, ['slot content'])
        root = render(el)

        self.assertIsInstance(root, Example)
        self.assertEqual(len(root.children), 1)
        self.assertEqual(root.children[0].tagName, 'DIV')
        self.assertEqual(len(root.children[0].children), 1)
        self.assertIsInstance(root.children[0].children[0], AnonymusElement)
        self.assertEqual(root.children[0].children[0].content, 'slot content')

    def test_slot_with_sibling(self):
        @Component(
            tag='kr-first',
            template=h('template', {}, [h('div', {}, ['div content']), h('slot', {}, [])])
        )
        class First(Element):
            pass

        el = h('kr-first', {}, ['slot content'])
        root = render(el)

        output = self.pprint(root, indent=None)
        self.assertEqual(output, '<KR-FIRST><DIV>div content</DIV>slot content</KR-FIRST>')

    def test_inherit_component_slot(self):
        @Component(
            tag='kr-first',
            template=h('template', {}, [h('div', {}, [h('slot', {}, [])])])
        )
        class First(Element):
            pass

        @Component(
            tag='kr-second',
            template=h('template', {}, [h('div', {}, ['div content']), h('kr-first', {}, [h('slot', {}, [])])])
        )
        class Second(Element):
            pass

        el = h('kr-second', {}, ['slot content'])
        root = render(el)

        self.assertEqual(root.tagName, 'KR-SECOND')
        self.assertEqual(len(root.children), 2)
        self.assertEqual(root.children[0].tagName, 'DIV')
        self.assertEqual(root.children[1].tagName, 'KR-FIRST')
        self.assertEqual(root.children[1].children[0].tagName, 'DIV')
        self.assertEqual(root.children[1].children[0].children[0].content, 'slot content')

    def test_inherited_slot_with_sibling(self):
        @Component(
            tag='kr-sidebar-main',
            template=self.require('''
            <template>
                <div class="container-fluid">
                    <slot />
                </div>
            </template>''')
        )
        class SidebarMain(Element):
            pass

        @Component(
            tag='kr-form-field',
            template=self.require('''<template>
                <div>
                    <label></label>
                    <kr-form-select>
                        <slot />
                    </kr-form-select>
                </div>
            </template>''')
        )
        class FormField(Element):
            @property
            def label_class(self):
                cls = 'col-form-label ' + ' '.join(f'col-{part}' for part in self.attrs.get('label-width', '').split(' '))
                if self.attrs.get('required', False):
                    cls += ' required'

                return cls

            @property
            def field_class(self):
                return ' '.join(f'col-{part}' for part in self.attrs.get('field-width', 'col').split(' '))

        sys.setrecursionlimit(500)

        el = h('kr-sidebar-main', {}, [h('kr-form-field', {'id': 'id_masodik', 'widget': 'kr-form-select', 'label-width': 'sm-4', 'field-width': 'sm-8', 'label': 'Próba választó'}, [h('option', {'value': 1}, ['One'])])])
        # root = next(render(el))
        root = render(el)
        print(self.pprint(root))



from browser import ajax, window, document


class Request:
    def _on_complete(self, request, callback):
        def complete(ev):
            if request.readyState != 4:
                return

            document.querySelector('#loading').style.display = 'none'
            if request.status == 340:
                window.location.replace(request.getResponseHeader('Location'))
            elif request.status in (200, 403):
                callback(request)

        return complete

    def _prepare(self):
        document.querySelector('#loading').style.display = 'block'

    def post(self, url, data, callback):
        self._prepare()
        req = window.XMLHttpRequest.new()
        req.open('POST', url)
        req.send(data)
        req.onreadystatechange = self._on_complete(req, callback)

    def get(self, url, callback):
        self._prepare()
        req = window.XMLHttpRequest.new()
        req.open('GET', url)
        req.send()
        req.onreadystatechange = self._on_complete(req, callback)


request = Request()



from pathlib import Path

from brython import list_modules
from brython.list_modules import ModulesFinder
from django.apps import apps
from django.core.management import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        stdlib_path = Path(apps.get_app_config('django_brython').path) / 'static/django_brython/js/brython_stdlib.js'
        brython_stdlib = list_modules.parse_stdlib(stdlib_dir=stdlib_path.parent, js_name='brython_stdlib.js')
        finder = ModulesFinder(str(Path.cwd() / 'proba'), stdlib=brython_stdlib)
        finder.inspect()

        # finder.modules.add('_warnings')
        # print('_warnings' in finder.modules)
        finder.make_brython_modules('static/django_brython/js/brython_modules.js')



from urllib.parse import urlparse

from django.conf import settings
from selenium import webdriver
from selenium.common.exceptions import JavascriptException, TimeoutException
from selenium.webdriver.support.wait import WebDriverWait


class TestFunctionRunner:
    driver: webdriver.Chrome
    UNITTEST_RESULT_VARNAME = '__BRYTHON_UNITTEST_RESULTS__'
    UNITTEST_RESULT_TIMEOUT = 60

    def __init__(self, test_obj, test_func, url, driver):
        self.test_obj = test_obj
        self.test_func = test_func

        parsed = urlparse(url)
        prefix = '?' if len(parsed.query) == 0 else '&'
        self.url = url + f'{prefix}__BRYTHON_UNITTEST__={test_func.__module__}.{test_func.__qualname__}'
        self.driver = getattr(self.test_obj, driver)
        # if driver not found raise error

    def unittest_globals_exists(self, driver):
        try:
            result = driver.execute_script(f"return __BRYTHON_UNITTEST_RESULTS__;")
            return result
        except JavascriptException:
            return False

    def __call__(self, *args, **kwargs):
        try:
            settings.BRYTHON_UNITTEST_RUNNING = True

            if not self.url.startswith('/'):
                self.url = '/' + self.url

            self.driver.get(f"{getattr(self.test_obj, 'live_server_url')}{self.url}")

            wait = WebDriverWait(self.driver, self.UNITTEST_RESULT_TIMEOUT, poll_frequency=0.1)

            try:
                test_result = wait.until(self.unittest_globals_exists)
            except TimeoutException as e:
                raise TimeoutException(f"Javascript global {self.UNITTEST_RESULT_VARNAME} isn't peresented in {self.UNITTEST_RESULT_TIMEOUT} seconds") from e

            if test_result['exception']:
                raise Exception(test_result['exception'])
        finally:
            settings.BRYTHON_UNITTEST_RUNNING = False


def location(url, driver='driver', wait_for=None, function=None, instance=None):
    if function is None and instance is None:
        def dec(func):
            return location(str(url), driver=driver, function=func, wait_for=wait_for)

        return dec

    if function is not None and instance is None:
        def dec(inst):
            runner = TestFunctionRunner(test_obj=inst, test_func=function, url=url, driver=driver)
            return runner()

        return dec


def reverse(*args, **kwargs):
    from django.urls import reverse

    return reverse(*args, **kwargs)


def reverse_lazy(*args, **kwargs):
    from django.urls import reverse_lazy

    return reverse_lazy(*args, **kwargs)



from browser import window

from kirui.core import createElement as h, Component, render, patch_dom_with_vdom, CustomElement
from kirui.utils.http import request
from django_brython.assets import require


@Component(
    tag='kr-form',
    template=require('form.html')
)
class Form(CustomElement):
    @property
    def form_data(self):
        form_data = window.FormData.new()
        form_data.append('csrfmiddlewaretoken', self.attrs['csrfmiddlewaretoken'])
        for field in self.querySelectorAll('[data-form-field]'):
            for k, v in field.form_data.items():
                if isinstance(v, (list, tuple)):
                    for v1 in v:
                        form_data.append(k, v1)
                else:
                    form_data.append(k, v)

        return form_data

    def _submit_complete(self, resp):
        if resp.status == 403:
            vel = eval(resp.responseText)
            patch_dom_with_vdom(self, vel.render())

    def submit(self, ev):
        ev.preventDefault()
        ev.stopPropagation()

        request.post(
            url=self.attrs['action'],
            data=self.form_data,
            callback=self._submit_complete
        )



from browser import window, document
from kirui.core import createElement as h, Component, patch_dom_with_vdom, CustomElement
from kirui.utils.http import request
from django_brython.assets import require


@Component(
    tag='kr-filtered-table',
    template=require('filtered_table.html')
)
class KrFilteredTable(CustomElement):
    def submit(self, ev):
        print('filtered-table-submit')
        print(ev.target)



import datetime

from browser import document, window
from kirui.core import createElement as h, Component, render, patch_dom_with_vdom, CustomElement
from kirui.utils.relativedelta import relativedelta
from django_brython.assets import require


@Component(
    tag='kr-date-picker',
    template=require('date_picker.html')
)
class DatePicker(CustomElement):
    @property
    def day_names(self):
        return ['Hét', 'Ked', 'Sze', 'Csü', 'Pén', 'Szo', 'Vas']

    @property
    def month_names(self):
        return ['Január', 'Február', 'Március', 'Április', 'Május', 'Június', 'Július', 'Augusztus', 'Szeptember', 'Október', 'November', 'December']

    @property
    def css_class(self):
        retval = {'parent': True, 'disabled': self.props['disabled']}
        if self.props.get('error', '') != '':
            retval['is-invalid'] = True

        return retval

    @property
    def is_readonly(self):
        if int(self.props['disabled']) == 1:
            return 'true'

        return ''

    @property
    def date_format(self):
        return self.props.get('date-format', '%Y-%m-%d')  # TODO: app beállításokból is kellene olvasni

    def step_month_backward(self, ev):
        ev.stopPropagation()
        ev.preventDefault()
        self.picker_date -= relativedelta(months=1)
        self._update_calendar_content()

    def step_month_forward(self, ev):
        ev.stopPropagation()
        ev.preventDefault()
        self.picker_date += relativedelta(months=1)
        self._update_calendar_content()

    def set_date(self, ev):
        ev.stopPropagation()
        ev.preventDefault()
        self.querySelector(f"input[name={self.props['name']}]").value = ev.target.getAttribute('data-value')
        self.value = datetime.datetime.strptime(ev.target.getAttribute('data-value'), self.date_format).date()
        self.hide_calendar(ev=None)

    def _generate_picker_content(self):
        iter_day = datetime.date(self.picker_date.year, self.picker_date.month, 1)

        content = h('div', {'class': 'date-picker-box', 'style': 'display: block'}, [
            h('div', {'class': 'paging'}, [
                h('div', {'class': 'material-icons', '{https://doculabs.io/2020/xtmpl#data-binding}onclick': lambda this: this.step_month_backward},
                  ['navigate_before']),
                h('div', {'class': 'month'}, [self.month_names[self.picker_date.month - 1]]),
                h('div', {'class': 'year'}, [str(self.picker_date.year)]),
                h('div', {'class': 'material-icons', '{https://doculabs.io/2020/xtmpl#data-binding}onclick': lambda this: this.step_month_forward},
                  ['navigate_next']),
            ]),
            h('div', {'class': 'day-names'}, [])
        ])

        for day_name in self.day_names:
            content.children[-1].children.append(h('div', {'class': 'day day-name'}, [day_name]))

        content.children.append(h('div', {'class': 'week'}, []))

        while True:
            v = iter_day.strftime(self.date_format)
            klass = {
                'day': True,
                'workday': iter_day.weekday() in [0, 1, 2, 3, 4],
                'weekend': iter_day.weekday() in [5, 6],
                'day-selected': v == self.querySelector(f"input[name={self.props['name']}]").value
            }

            # A naptár elején üres napok vannak, amik az előző hónapból "nyúlnak át"
            if iter_day.day == 1:
                style = {'margin-left': f'{44*iter_day.weekday()+2}px'}
            else:
                style = {}

            content.children[-1].children.append(
                h('div', {
                    'class': klass,
                    'data-value': v,
                    '{https://doculabs.io/2020/xtmpl#data-binding}onclick': lambda this: this.set_date,
                    'style': style
                }, [str(iter_day.day)]))
            if iter_day.weekday() == 6:
                content.children.append(h('div', {'class': 'week'}, []))
            iter_day = iter_day + datetime.timedelta(days=1)
            if self.picker_date.month != iter_day.month:
                break

        return content

    def _update_calendar_content(self):
        content = self._generate_picker_content()
        content = content.render(component=self)
        patch_dom_with_vdom(self.querySelector('.date-picker-box'), content)

    def show_calendar(self, ev):
        if ev:
            ev.stopPropagation()
            ev.preventDefault()

        # ha le van tiltva a mező, akkor ne is jelenítsük meg a naptárat
        if self.props.get('disabled', '0') == '1':
            return

        self.picker_date = self.value or datetime.date.today()
        self.picker_date = datetime.date(self.picker_date.year, self.picker_date.month, 1)
        self._update_calendar_content()
        self.querySelector('.date-picker-box').style['display'] = 'block'

    def hide_calendar(self, ev):
        if ev:
            parent = ev.target
            while True:
                if parent == document.body or parent == self:
                    break

                try:
                    parent = parent.parent
                except AttributeError:
                    break

            if parent == self:
                return

        # Ha a felhasználó kitörölte a mező tartalmát, akkor nullázzuk az értéket
        if len(self.querySelector(f"input[name={self.props['name']}]").value.strip()) == 0:
            self.value = ''

        self.querySelector('.date-picker-box').style['display'] = 'none'

    def connectedCallback(self):
        self.querySelector('input').addEventListener('click', self.show_calendar)
        document.addEventListener('click', self.hide_calendar)

        self.value = self.props.get('value', '')
        if len(self.value):
            self.value = datetime.datetime.strptime(self.value, self.date_format).date()

    @property
    def form_data(self):
        field = self.querySelector('input')
        return {field.attrs['name']: field.value}



from .forms import Form  # NOQA
from .fields import FormField  # NOQA
from .widgets.select import MultiSelectCheckbox, Select, OptionCheckbox # NOQA
from .widgets.input import Input # NOQA
from .widgets.date import DatePicker  # NOQA
from .widgets.checkbox import CheckboxSwitch  # NOQA
from .widgets.text import TextBox  # NOQA
from .widgets.file import FileUpload  # NOQA



from browser import document, webcomponent
from kirui.core import constants
from kirui.core.events import event_handler


class CustomElement:
    def render(self, children=[]):
        return self.component_config.template.render(component=self, slot_content=children)

    def connectedCallback(self):
        pass


class createElement:
    def __init__(self, tag, props, children):
        self.tag = tag
        self.props = props
        self.children = children

    def __repr__(self):
        attrs = ' '.join(f'{k}="{v}"' for k, v in self.props.items())
        return f'<{self.tag} {attrs}>'

    def render(self, slot_content=[], component=None):
        klass = webcomponent.get(self.tag)
        component_config = getattr(klass, 'component_config', None)
        if component_config is None:
            element = document.createElement(self.tag)

            for k, v in self.props.items():
                if self.tag == 'template':  # TODO: ennél robosztusabb megoldást a komponens sablon felismerésre
                    continue

                if k.startswith('{https://doculabs.io/2020/xtmpl#data-binding}'):
                    k = k.replace('{https://doculabs.io/2020/xtmpl#data-binding}', '')
                    if k in constants.JAVASCRIPT_EVENT_NAMES:
                        element.addEventListener(k.replace('on', ''), event_handler(v))
                        continue
                    else:
                        v = v(component or element)

                if k == 'class' and isinstance(v, dict):
                    v = ' '.join(ck for ck, cv in v.items() if cv is True)
                elif k == 'style' and isinstance(v, dict):
                    v = ';'.join(f'{ck}: {cv}' for ck, cv in v.items())
                elif k == 'text':
                    element.appendChild(document.createTextNode(v))

                if (isinstance(v, str) and len(v) <= 0) or k.startswith('{https://doculabs.io/2020/xtmpl#control}'):
                    continue

                if v is None:  # None érték -> üres stinrg
                    v = ''
                element.setAttribute(k, v)

            for vchild in self.children:
                if isinstance(vchild, str):
                    element.appendChild(document.createTextNode(vchild))
                elif vchild.tag == 'slot':
                    for vc in slot_content:
                        if isinstance(vc, str):
                            element.appendChild(document.createTextNode(vc))
                        else:
                            element.appendChild(vc.render(component=component, slot_content=[]))
                else:
                    if v := vchild.props.get('{https://doculabs.io/2020/xtmpl#control}if', None):
                        v = v(component)
                        if v is False:
                            continue

                    element.appendChild(vchild.render(component=component, slot_content=slot_content))
        else:
            element = document.createElement(self.tag)

            # evaluate properties
            element.props = {}
            for k, v in self.props.items():
                if k.startswith('{https://doculabs.io/2020/xtmpl#data-binding}'):
                    k = k.replace('{https://doculabs.io/2020/xtmpl#data-binding}', '')
                    v = v(component or element)

                element.props[k] = v

            # set element attributes according to template
            for k, v in component_config.template.props.items():
                if k.startswith('{https://doculabs.io/2020/xtmpl#data-binding}'):
                    k = k.replace('{https://doculabs.io/2020/xtmpl#data-binding}', '')

                element.setAttribute(k, element.props.get(k, v))

            root = element.render(children=slot_content or self.children)  # template
            for child in root.childNodes:
                element.appendChild(child)

        return element



from kirui.core import createElement as h, Component, CustomElement
from django_brython.assets import require


@Component(
    tag='kr-file',
    template=require('input.html')
)
class FileUpload(CustomElement):
    @property
    def css_class(self):
        retval = {'form-control': True}
        if self.props.get('error', '') != '':
            retval['is-invalid'] = True

        return retval

    @property
    def is_readonly(self):
        if int(self.props['disabled']) == 1:
            return 'true'

        return ''

    @property
    def input_type(self):
        return 'file'

    @property
    def form_data(self):
        field = self.querySelector('input[type=file]')
        retval = {field.attrs['name']: ''}
        if len(field.files) > 0:
            retval[field.attrs['name']] = field.files[0]

        return retval



from browser import document, window
from kirui.core import createElement as h, Component, CustomElement
from django_brython.assets import require

jq = window.jQuery


@Component(
    tag='kr-multi-select-checkbox',
    template=require('multi_select_checkbox.html')
)
class MultiSelectCheckbox(CustomElement):
    def toggle_dropdown(self, ev):
        ev.stopPropagation()
        if (ev.target.tagName in {'INPUT', 'LABEL'}):
            return

        jq(self).find('.dropdown').toggleClass('show')

    def update_labels(self, ev):
        values = ', '.join(cb.parent.text for cb in self.querySelectorAll('input[type=checkbox]') if cb.checked)
        el = self.querySelectorAll('.select-value')[0]
        el.text = values or 'Kérlek válassz...'

    def connectedCallback(self):
        if self.props['disabled'] != '1':
            jq(self).find('.parent').on('click', self.toggle_dropdown)
            jq(self).find('input[type=checkbox]').on('change', self.update_labels)

        self.update_labels(None)

    @property
    def css_class(self):
        retval = {'parent': True, 'readonly': self.props['disabled'] == '1'}
        if self.props.get('error', '') != '':
            retval['is-invalid'] = True

        return retval

    @property
    def form_data(self):
        retval = {self.props['name']: []}
        options = self.querySelectorAll('input[type=checkbox]')

        for option in options:
            if getattr(option, 'checked', 'true'):
                retval[self.props['name']].append(option.attrs['value'])

        return retval


@Component(
    tag='kr-option-checkbox',
    template=require('option_checkbox.html')
)
class OptionCheckbox(CustomElement):
    def connectedCallback(self):
        #print(self.props)
        pass


import tempfile
from importlib import import_module

from django import apps

from .brython import make_package
from django.conf import settings
from django.contrib.staticfiles.finders import BaseFinder
from pathlib import Path

from django.core.exceptions import ImproperlyConfigured
from django.core.files import File
from django.core.files.storage import Storage


class TempDirStorage(Storage):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.directory = tempfile.TemporaryDirectory()

    def path(self, name):
        return f'{self.directory.name}/{name}'

    def _open(self, name, mode):
        name = name.split('/')[-1]  # Don't handle directory structure in tempdir
        return File(open(f'{self.directory.name}/{name}', mode))


class BrythonStaticGenerator(BaseFinder):
    def __init__(self, app_names=None, *args, **kwargs):
        module_path = Path(__file__).parent

        fronted_app = getattr(settings, 'BRYTHON_BUNDLER_MAIN_MODULE', None)
        if not fronted_app:
            raise ImproperlyConfigured('Please setup BRYTHON_BUNDLER_MAIN_MODULE')
        else:
            self.brython_module_path = Path(import_module(fronted_app).__file__).parent
            # fronted_app = import_string(fronted_app)

        self.storage = TempDirStorage()
        # stdlib_path = Path(apps.get_app_config('django_brython').path) / 'static/django_brython/js/brython_stdlib.js'
        # brython_stdlib = list_modules.parse_stdlib(stdlib_dir=stdlib_path.parent, js_name='brython_stdlib.js')
        # user_modules = list_modules.load_user_modules(module_dir=module_path)
        # print(user_modules)

        # self.brython_user_module = brython_user_module



        # stdlib_dir, stdlib = list_modules.load_stdlib_sitepackages()

        path = Path.cwd() / 'frontend'

        """finder.make_brython_modules()
        print(finder.modules)"""

    def list(self, ignore_patterns):
        files = []

        """user_modules = list_modules.load_user_modules(self.brython_module_path)
        finder = list_modules.ModulesFinder(directory=str(self.brython_module_path), stdlib={}, user_modules=user_modules)
        finder.inspect()
        print(finder.modules)
        finder.make_brython_modules(f'{self.storage.directory.name}/{self.brython_module_path.name}')"""

        exclude_modules = getattr(settings, 'BRYTHON_BUNDLER_EXCLUDE', [])
        with open(f'{self.storage.directory.name}/{self.brython_module_path.name}.brython.js', 'w', encoding='utf-8') as f:
            make_package.make(package_name=self.brython_module_path.name,
                              package_path=str(self.brython_module_path),
                              output=f,
                              exclude_modules=exclude_modules)

        files.append((f'{self.brython_module_path.name}/js/{self.brython_module_path.name}.brython.js', self.storage))

        return files

    def find(self, path, all=False):
        p = Path(self.storage.directory.name) / path
        p.parent.mkdir(parents=True, exist_ok=True)

        exclude_modules = getattr(settings, 'BRYTHON_BUNDLER_EXCLUDE', [])
        with p.open('w', encoding='utf-8') as f:
            make_package.make(package_name=self.brython_module_path.name,
                              package_path=str(self.brython_module_path),
                              output=f,
                              exclude_modules=exclude_modules)

        return p



from django.urls import reverse
from django.utils.deprecation import MiddlewareMixin
from django.utils.module_loading import import_string


class BrythonUnittestMiddleware(MiddlewareMixin):
    TEST_CODE_TEMPLATE = """
    <script type="text/python">
        import traceback, sys
        from importlib import reload
        from browser import document, window

        def run_test(event=None):
            import {module}
            reload({module})
            from {module} import {klass}
            test = {klass}()
            func = getattr(test, '{func_name}')
            func()
            print("test finished")

        run_test()

        document.bind('reload_brython_module@after', run_test)
    </script>
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        self.process_template_response(request, response)

        return response

    def process_template_response(self, request, response):
        unittest = request.GET.get('__BRYTHON_UNITTEST__', None)

        if unittest is None:
            return response

        try:
            klass, function = unittest.rsplit('.', 1)
        except ValueError as e:
            raise ImportError(f'Please specify the full path of the test method') from e

        klass = import_string(klass)

        test_html = self.TEST_CODE_TEMPLATE.format(
            module=klass.__module__,
            klass=klass.__name__,
            func_name=function
        ).encode('utf-8')

        response.content += test_html
        return response



from kirui.core import createElement as h, Component, CustomElement
from django_brython.assets import require


@Component(
    tag='kr-checkbox-switch',
    template=require('checkbox_switch.html')
)
class CheckboxSwitch(CustomElement):
    @property
    def css_class(self):
        retval = {'form-control': True, 'form-check-input': True}
        if self.props.get('error', '') != '':
            retval['is-invalid'] = True

        return retval

    @property
    def is_disabled(self):
        if int(self.props['disabled']) == 1:
            return 'disabled'

        return ''

    @property
    def is_checked(self):
        if self.props.get('value', '') in ('true', True, 1):
            return '1'

        return ''

    @property
    def form_data(self):
        return {self.props['name']: self.querySelector('input[type=checkbox]').checked}



from kirui.core import createElement as h, Component, CustomElement
from django_brython.assets import require


@Component(
    tag='kr-select',
    template=require('select.html')
)
class Select(CustomElement):
    @property
    def css_class(self):
        retval = {'form-select': True}
        if self.attrs.get('error', '') != '':
            retval['is-invalid'] = True

        return retval

    @property
    def form_data(self):
        return {self.props['name']: self.querySelector('select').value}



from kirui.core import createElement as h, Component, CustomElement
from django_brython.assets import require


@Component(
    tag='kr-form-field',
    template=require('fields.html')
)
class FormField(CustomElement):
    @property
    def label_class(self):
        cls = 'col-form-label ' + ' '.join(f'col-{part}' for part in self.props.get('label-width', '').split(' '))
        if self.props.get('required', 'false') in ('true', True, 1):
            cls += ' required'

        return cls

    @property
    def field_class(self):
        return ' '.join(f'col-{part}' for part in self.props.get('field-width', 'col').split(' '))





class Application:
    def __init__(self, is_pwa: bool = False):
        self.is_pwa = is_pwa
        self.components = set()

    def register_component(self, component):
        self.components.add(component)
        component.component_config.application = self



from browser import window, document
from django_brython.assets import require as req
from kirui.core import createElement as h, Component, CustomElement

jq = window.jQuery


@Component(
    tag='kr-layout-sidebar',
    template=req('sidebar.html')
)
class SidebarLayout(CustomElement):
    _is_sidebar_toggled: bool
    _is_sidebar_pinned: bool
    _sidebar_is_moving: bool
    sidebar_width: int

    def __init__(self, *args, **kwargs):
        self._sidebar_width = 256
        self._is_sidebar_pinned = True
        self._is_sidebar_toggled = False
        self._sidebar_is_moving = False
        self.counter = 10

    @property
    def sidebar_width(self):
        return f'{self._sidebar_width}px'

    def expand_menu(self, ev):
        ev.preventDefault()
        ev.stopPropagation()
        for submenu in ev.target.closest('li').querySelectorAll('ul'):
            if submenu.classList.contains('expanded'):
                submenu.classList.remove('expanded')
            else:
                submenu.classList.add('expanded')

    def navbar_pinner_clicked(self, ev):
        if self.component_config.application.is_pwa:
            if self._is_sidebar_toggled:
                jq(self).find('kr-sidebar-nav').css({'left': f'-{self.sidebar_width}'})
                jq(self).find('#navbar-overflow').css({'display': 'none'})
                self._is_sidebar_toggled = False
            else:
                jq(self).find('kr-sidebar-nav').css({'left': '0'})
                jq(self).find('#navbar-overflow').css({'display': 'block'})
                self._is_sidebar_toggled = True
        else:
            if self._is_sidebar_pinned:
                jq(self).find('kr-sidebar-nav').css({'left': f'-{self.sidebar_width}'})
                jq(self).find('kr-sidebar-main').css({'margin-left': '0'})
                self._is_sidebar_pinned = False
            else:
                jq(self).find('kr-sidebar-nav').css({'left': '0'})
                jq(self).find('kr-sidebar-main').css({'margin-left': f'{self.sidebar_width}'})
                self._is_sidebar_pinned = True

    def navbar_touch_start(self, ev):
        jq(self).find('kr-sidebar-nav').css({'transition': 'all 0s ease'})
        x_coord = ev.touches[0].pageX
        if self._is_sidebar_toggled is False and x_coord < 50:
            ev.preventDefault()
            self._sidebar_is_moving = True
        elif self._is_sidebar_toggled and self._sidebar_width - 50 <= x_coord <= self._sidebar_width + 50:
            self._sidebar_is_moving = True

    def navbar_touch_move(self, ev):
        if self._sidebar_is_moving is False:
            return

        left = int(ev.touches[0].pageX) - self._sidebar_width
        if left > 0:
            left = 0
        jq(self).find('kr-sidebar-nav').css({'left': f'{left}px'})

    def navbar_touch_end(self, ev):
        if self._sidebar_is_moving is False:
            return

        jq(self).find('kr-sidebar-nav').css({'transition': 'all 0.5s ease'})
        if ev.changedTouches[0].pageX >= 256 / 2:
            jq(self).find('kr-sidebar-nav').css({'left': '0'})
            self._is_sidebar_toggled = True
            jq(self).find('#navbar-overflow').css({'display': 'block'})
        else:
            jq(self).find('kr-sidebar-nav').css({'left': f'-{self.sidebar_width}'})
            self._is_sidebar_toggled = False
            jq(self).find('#navbar-overflow').css({'display': 'none'})

        self._sidebar_is_moving = False

    def show_navbar(self, ev):
        ev.preventDefault()
        if self._is_sidebar_pinned is False:
            jq(self).find('kr-sidebar-nav').css({'left': '0'})
            self._is_sidebar_toggled = True

    def hide_navbar(self, ev):
        ev.preventDefault()
        if self._is_sidebar_toggled:
            self._is_sidebar_toggled = False
            jq(self).find('kr-sidebar-nav').css({'left': f'-{self.sidebar_width}'})

    def dropdown_toggle(self, ev):
        ev.preventDefault()
        ev.stopPropagation()

        parent_rect = ev.target.parent.getBoundingClientRect()
        jq(ev.target).addClass('show')
        for el in ev.target.parent.getElementsByClassName('dropdown-menu'):
            if el.classList.contains('show'):
                el.classList.remove('show')
            else:
                el.classList.add('show')
                rect = el.getBoundingClientRect()
                offset_left = int(parent_rect.width - rect.width)
                el.style.left = f'{offset_left}px'
                el.style.top = f'{parent_rect.top + parent_rect.height - 7}px'

    def window_click(self, ev):
        jq('.show').removeClass('show')

    def connectedCallback(self):
        jq(self).find('kr-sidebar-main, kr-sidebar-nav').on('touchstart', self.navbar_touch_start)
        jq(self).find('kr-sidebar-main, kr-sidebar-nav').on('touchmove', self.navbar_touch_move)
        jq(self).find('kr-sidebar-main, kr-sidebar-nav').on('touchend', self.navbar_touch_end)
        jq(self).find('#navbar-pinner').click(self.navbar_pinner_clicked)
        jq(self).find('.dropdown-toggle').on('click', self.dropdown_toggle)
        jq(window).on('click', self.window_click)

        for sp in self.querySelectorAll('kr-sidebar-nav .expand'):
            sp.addEventListener('click', self.expand_menu)

        for main_menu in self.querySelectorAll('kr-sidebar-nav > ul > li > a'):
            if window.location.pathname.startswith(main_menu.attrs['href']):
                main_menu.classList.add('active')
                if sub_menu := main_menu.parent.querySelector('.collapse'):
                    sub_menu.classList.add('expanded')

        if self.component_config.application.is_pwa is False:
            jq(self).find('#navbar-toggler').on('mouseover', self.show_navbar)
            jq(self).find('kr-sidebar-nav').on('mouseleave', self.hide_navbar)



@Component(
    tag='kr-sidebar-main',
    template=req('sidebar_main.html')
)
class SidebarMain(CustomElement):
    def __init__(self):
        self.counter = 8

    def connectedCallback(self):
        pass


@Component(
    tag='kr-sidebar-header',
    template=req('sidebar_header.html')
)
class SidebarHeader(CustomElement):
    def connectedCallback(self):
        pass


@Component(
    tag='kr-sidebar-nav',
    template=req('sidebar_nav.html')
)
class SidebarNavigation(CustomElement):
    def connectedCallback(self):
        pass



import ast
import json
import os
import sys
import time
from io import StringIO
from pathlib import Path
from typing import Any

from samon.environment import Environment
from samon.parser import DefaultParser
from . import python_minifier


class Visitor(ast.NodeVisitor):
    """Used to list all the modules imported by a script."""

    def __init__(self, lib_path, package):
        self.imports = set()
        self.lib_path = lib_path
        self.package = package

    def visit_Import(self, node):
        for alias in node.names:
            self.imports.add(alias.name)

    def visit_ImportFrom(self, node):
        if node.level > 0:
            package = self.package[:]
            level = node.level - 1
            while level:
                package.pop()
                level -= 1
            module = ".".join(package)
            if node.module:
                module += "." + node.module
        else:
            module = node.module
        self.imports.add(module)
        for alias in node.names:
            if alias.name == "*":
                continue
            else:
                # Only keep "from X import Y" if X.Y is a module, not if Y
                # is a variable defined in X
                path = os.path.join(self.lib_path, *module.split("."),
                    alias.name + ".py")
                if os.path.exists(path):
                    self.imports.add(module + "." + alias.name)


class BrythonTransformer(ast.NodeTransformer):
    def __init__(self, source_path):
        self.path = source_path
        self.require_function = None

    def visit_ImportFrom(self, node: ast.ImportFrom) -> Any:
        if node.module == 'django_brython.assets':
            for nname in node.names:
                if nname.name == 'require':
                    self.require_function = nname.asname or nname.name
                    return None

        return node

    def visit_Import(self, node: ast.Import) -> Any:
        for nname in node.names:
            if nname.name == 'django_brython.assets.require':
                self.require_function = nname.asname or nname.name
                return None

        return node

    def generic_visit(self, node: ast.AST) -> ast.AST:
        if isinstance(node, ast.Import):
            return self.visit_Import(node)
        elif isinstance(node, ast.ImportFrom):
            return self.visit_ImportFrom(node)
        elif isinstance(node, ast.Call):
            if hasattr(node.func, 'id') and node.func.id == self.require_function:
                template_path = Path(node.args[0].value)
                if not template_path.is_absolute():
                    template_path = (self.path.parent / node.args[0].value).resolve()
                with template_path.open('r', encoding='utf-8') as f:
                    parser = DefaultParser(environment=Environment(loader=None))
                    template = parser.parse(source=f.read(), template_name=template_path.name)
                    return template.serialize(output='psx')

        return super().generic_visit(node)


def make(package_name, package_path, exclude_dirs=None, output: StringIO=None, exclude_modules=[]):
    if not package_name:
        raise ValueError("package name is not specified")
    # print("Generating package {}".format(package_name))
    VFS = {"$timestamp": int(1000 * time.time())}
    has_init = os.path.exists(os.path.join(package_path, "__init__.py"))
    nb = 0
    if exclude_dirs is None:
        exclude_dirs = []

    if hasattr(ast, 'unparse'):
        ast_unparse_working = True
    else:
        ast_unparse_working = False
        print("ast module hasn't 'unparse' method, bundling assets is not working (require python >= 3.9)", file=sys.stderr)

    for dirpath, dirnames, filenames in os.walk(package_path):
        flag = False
        root_elts = dirpath.split(os.sep)
        for exclude in exclude_dirs:
            if exclude in root_elts:
               continue
        if '__pycache__' in dirnames:
            dirnames.remove("__pycache__")

        if dirpath == package_path:
            package = []
        else:
            package = dirpath[len(package_path) + 1:].split(os.sep)
        if has_init:
            package.insert(0, package_name)

        for filename in filenames:
            name, ext = os.path.splitext(filename)
            if ext != '.py':
                continue
            is_package = name.endswith('__init__')
            if is_package:
                mod_name = '.'.join(package)
            else:
                mod_name = '.'.join(package + [name])

            nb += 1
            absname = os.path.join(dirpath, filename)
            with open(absname, encoding='utf-8') as f:
                src = f.read()

            tree = ast.parse(src)

            if ast_unparse_working:
                src = ast.unparse(BrythonTransformer(source_path=Path(absname)).visit(tree).body)

            data = python_minifier.minify(src, preserve_lines=True)
            path_elts = package[:]
            if os.path.basename(filename) != "__init__.py":
                path_elts.append(os.path.basename(filename)[:-3])

            visitor = Visitor(package_path, package)
            visitor.visit(tree)  # todo: ez nem a transzformált fa
            imports = sorted(list(visitor.imports))

            for exc_mod_name in exclude_modules:
                if mod_name.startswith(exc_mod_name):
                    break
            else:
                if is_package:
                   VFS[mod_name] = [ext, data, imports, 1]
                else:
                    VFS[mod_name] = [ext, data, imports]

            # print("adding {} package {}".format(mod_name, is_package))

    if nb == 0:
        print("No Python file found in current directory")
    else:
        # print('{} files'.format(nb))

        output = output or StringIO()
        # output_path = output_path or os.path.join(package_path, package_name + ".brython.js")
        #with open(output_path, "w", encoding="utf-8") as out:
        output.write('__BRYTHON__.use_VFS = true;\n')
        output.write('var scripts = {}\n'.format(json.dumps(VFS)))
        output.write('__BRYTHON__.update_VFS(scripts)\n')
        output.seek(0)

        return output


if __name__ == "__main__":
    import sys
    package_name = sys.argv[1] if len(sys.argv) > 1 else ""
    src_dir = sys.argv[2] if len(sys.argv) > 2 else os.getcwd()

    make(package_name, src_dir)



from browser import document

from .elements import createElement


def render(vel: 'createElement', slot_content=[], component=None, fix_component=False):
    return vel.render()

    """element = document.createElement(vel.tag)

    if hasattr(element, 'component_config') and component is None:
        component = element

    for k, v in vel.props.items():
        if k.startswith('{https://doculabs.io/2020/xtmpl#data-binding}'):
            k = k.replace('{https://doculabs.io/2020/xtmpl#data-binding}', '')
            if k not in {'onclick', 'onsubmit',}:
                v = v(component)

        if k == 'class' and isinstance(v, dict):
            v = ' '.join(ck for ck, cv in v.items() if cv is True)
        elif k == 'style' and isinstance(v, dict):
            v = ';'.join(f'{ck}: {cv}' for ck, cv in v.items())
        elif k == 'text':
            element.appendChild(document.createTextNode(v))

        if (isinstance(v, str) and len(v) <= 0) or k.startswith('{https://doculabs.io/2020/xtmpl#control}'):
            continue

        if k in {'onclick', 'onsubmit',}:
            def handle_wrapper(fn, component):
                def handler(ev):
                    fn2 = fn(self=ev.target)
                    if callable(fn2):
                        return fn2(ev)
                    else:
                        return fn2

                return handler

            element.addEventListener(k.replace('on', ''), handle_wrapper(v, component))
        else:
            if v is None:  # None érték -> üres stinrg
                v = ''
            element.setAttribute(k, v)

    # Ha a komponenst fixáltam, akkor nem szeretném lecserélni
    if hasattr(element, 'component_config') and fix_component is False:
        component = element

    if hasattr(element, 'component_config'):  # it is a custom component
        slot_content = slot_content or vel.children
        rendered = render(element.component_config.template, slot_content=slot_content, component=component)
        for child in rendered.children:
            element.appendChild(child)
    else:
        if vel.tag == 'slot':
            for child in slot_content:
                if isinstance(child, str):
                    element.appendChild(document.createTextNode(child))
                else:
                    element.appendChild(render(child, slot_content=[], component=component))
        else:
            for child in vel.children:
                if isinstance(child, str):
                    element.appendChild(document.createTextNode(child))
                else:
                    if v := child.props.get('{https://doculabs.io/2020/xtmpl#control}if', None):
                        v = v(component)
                        if v is False:
                            continue

                    rendered = render(child, slot_content=slot_content, component=component)
                    if rendered.tagName == 'SLOT':
                        for child2 in rendered.childNodes:
                            element.appendChild(child2)
                    else:
                        element.appendChild(rendered)

    return element"""



from browser import ajax, webcomponent, window, document
from django_brython.assets import require
from kirui.application import Application
from kirui.components.layout.sidebar import SidebarLayout, SidebarMain, SidebarHeader, SidebarNavigation
from kirui.components.forms import Form, FormField, Select, Input, CheckboxSwitch, OptionCheckbox, DatePicker, TextBox, FileUpload
from kirui.components.modal import ModalLink, SideModal, ModalHeader, ModalBody, ModalFooter
from kirui.components.table import KrTable, KrFilteredTable

from kirui.core import createElement as h, patch_dom_with_vdom, render  # NOQA


def entrypoint(is_pwa, template, **kwargs):
    app = Application(is_pwa=bool(int(is_pwa)))
    for component in [SidebarLayout, SidebarMain, SidebarHeader, SidebarNavigation]:
        app.register_component(component)

    for component in [Form, FormField, Select, Input, CheckboxSwitch, OptionCheckbox, DatePicker, TextBox, FileUpload]:
        app.register_component(component)

    for component in [ModalLink, SideModal, ModalHeader, ModalBody, ModalFooter]:
        app.register_component(component)

    for component in [KrTable, KrFilteredTable]:
        app.register_component(component)

    patch_dom_with_vdom(document.getElementById('app'), template.render())



import datetime
import time

from django.http import HttpResponse, JsonResponse, HttpResponseRedirect
from django.shortcuts import render
from django.core.cache import cache
from django.template.loader import get_template
from django.views import View

from django_kirui.utils.paginator import Paginator
from kirui_devserver.backend.forms import SampleForm, FilterForm
from kirui_devserver.backend.models import Activity


def index(request):
    return render(request, 'xml/index.html')


class FormView(View):
    def get(self, request):
        form = SampleForm(request.POST or None, request.FILES or None)
        return render(request, 'xml/form.html', context={'form': form})

    def post(self, request):
        form = SampleForm(request.POST or None, request.FILES or None, instance=Activity.objects.first())
        if form.is_valid():
            resp = HttpResponseRedirect('/backend/index/')
            resp.status_code = 340
            return resp
        else:
            data = get_template('xml/form_data.html').render(context={'form': form}, request=request, to='psx')
            resp = HttpResponse(data)
            resp.status_code = 403
            return resp


def modal(request):
    form = SampleForm(request.POST or None, request.FILES or None)
    form.now = datetime.datetime.now()
    if form.is_valid():
        return HttpResponse('OK')

    data = get_template('xml/modal.html').render(context={'form': form}, request=request, to='psx')
    resp = HttpResponse(data)
    if form.is_valid():
        resp.status_code = 200
    else:
        resp.status_code = 403

    return resp


def table(request):
    data = []
    for row in range(1, 20):
        data.append({'first': f'{row}.1', 'second': f'{row}.2'})

    paginate_to = int(request.GET.get('paginate_to', 1))
    p = Paginator(data, 4)
    page = p.page(paginate_to)

    if request.GET.get('paginate_to', None):
        #time.sleep(2)
        data = get_template('xml/table_data.html').render(context={'page': page}, request=request, to='psx')
        return HttpResponse(data)
    else:
        return render(request, 'xml/table.html', context={'page': page})


class FilteredTable(View):
    def dispatch(self, request, *args, **kwargs):
        data = []
        for row in range(1, 20):
            data.append({'first': f'{row}.1', 'second': f'{row}.2'})
        self.data = data
        return super().dispatch(request, *args, **kwargs)

    def get(self, request):
        form = FilterForm()
        p = Paginator(self.data, 4)
        page = p.page(1)
        return render(request, 'xml/filtered_table.html', context={'form': form, 'page': page})



from browser import window, document
from kirui.core import createElement as h, Component, patch_dom_with_vdom, CustomElement
from kirui.utils.http import request
from django_brython.assets import require


@Component(
    tag='kr-table',
    template=require('table.html')
)
class KrTable(CustomElement):
    def paginate(self, ev):
        def finished(resp):
            if resp.status == 200:
                vel = eval(resp.responseText)
                patch_dom_with_vdom(self, vel.render())
                # self.querySelector('kr-paginator tr td').setAttribute('colspan', self.column_count)

        request.get(f"?paginate_to={ev.target.getAttribute('data-page-to')}", callback=finished)

    def connectedCallback(self):
        self.column_count = len(self.querySelectorAll('kr-columns kr-column'))
        self.querySelector('kr-paginator tr td').setAttribute('colspan', self.column_count)


@Component(
    tag='kr-paginator',
    template=require('paginator.html')
)
class KrPaginator(CustomElement):
    @property
    def prev_page_class(self):
        return {'page-item': True, 'disabled': self.props['prevpage'] == ''}

    @property
    def next_page_class(self):
        return {'page-item': True, 'disabled': self.props['nextpage'] == ''}



from browser import document, window

from kirui.core import createElement as h, Component, render, patch_dom_with_vdom, CustomElement
from kirui.utils.http import request
from django_brython.assets import require


@Component(
    tag='kr-modal-link',
    template=require('modal_link.html')
)
class ModalLink(CustomElement):
    def _add_event_listeners(self):
        for btn in self.querySelector('kr-modal-header').querySelectorAll('.btn-close'):
            btn.addEventListener('click', self.close_modal)

        for btn in self.querySelector('kr-modal-footer').querySelectorAll('input,button'):
            btn.addEventListener('click', self.submit)

    def _init_modal(self, resp):
        data = eval(resp.responseText)
        el = render(data, component=self)
        self.appendChild(el)
        self._add_event_listeners()
        document.body.classList.add('modal-open')

    def open_modal(self, ev):
        ev.preventDefault()
        ev.stopPropagation()

        request.get(
            url=self.querySelector('a').attrs['href'],
            callback=self._init_modal
        )

    def close_modal(self, ev):
        ev.preventDefault()
        ev.stopPropagation()
        self.querySelector('kr-side-modal').remove()
        document.body.classList.remove('modal-open')

    def _submit_complete(self, resp):
        if resp.status == 403:
            data = eval(resp.responseText)
            el = render(data, component=self)
            patch_dom_with_vdom(self.querySelector('kr-side-modal'), el)
            self._add_event_listeners()
        elif resp.status == 200:
            self.close_modal(window.CustomEvent.new())

    def submit(self, ev):
        ev.preventDefault()
        ev.stopPropagation()

        form = self.querySelector('kr-form')
        request.post(
            url=form.attrs['action'],
            data=form.form_data,
            callback=self._submit_complete
        )


@Component(
    tag='kr-side-modal',
    template=require('side_modal.html')
)
class SideModal(CustomElement):
    @property
    def css_style(self):
        return {
            'width': self.attrs.get('width', '50%')
        }


@Component(
    tag='kr-modal-header',
    template=require('modal_header.html')
)
class ModalHeader(CustomElement):
    pass


@Component(
    tag='kr-modal-body',
    template=require('modal_body.html')
)
class ModalBody(CustomElement):
    pass


@Component(
    tag='kr-modal-footer',
    template=require('modal_footer.html')
)
class ModalFooter(CustomElement):
    pass
